from enum import Enum
import ams_visa_scpi
import time
from time import sleep
import sys
class ams_temperature_control:
    
    """
    This class defines the interface of controlling thermostreams
    """
    def __init__(self, address: str="GPIB1::1::INSTR"):
        """
        This function initialises the connection to the instrument.
        """
        self.connection = ams_visa_scpi.VisaScpi(address)

    def headLock(self):
        """
        This function makes the head of the thermostream unable to move.
        """
        self.connection.writeRaw("HDLK 1")

    def headUnlock(self):
        """
        This function makes the head of the thermostream able to be moved
        """
        self.connection.writeRaw("HDLK 0")

    def headUp(self) -> bool:
        """
        This function moves the head of the thermostream up.

        Returns:
            bool: this is true if the command is executed without problem.
        """
        self.connection.writeRaw("HEAD 0")

    def headDown(self) -> bool:
        """
        This function moves the head of the thermostream down.

        Returns:
            bool: this is true if the command is executed without problem.
        """
        self.connection.writeRaw("HEAD 1")

    def setSectionPoint(self, setPoint: int) -> bool:
        """
        This function sets the section point of the thermostream accordingly to the given temperature.

        Args:
            setPoint (int): type of the section, with the respect due to temperature(1, 2 or 3) 

        Returns:
            bool: this is true if command is executed without problem.
        """
        self.connection.writeRaw("SETN " + str(setPoint))

    def readTemp(self) -> bool:
        """
        This function reads the temperature from the thermostream.

        Returns:
            bool: this is true if command is executed without problem.
        """
        self.connection.writeRaw("TEmp?")
        time.sleep(0.1)
        return float(self.connection.readRaw().strip())

    def airFlowOn(self) -> bool:
        """
        This function enables the flowing of the air from the thermostream.

        Returns:
            bool: this is true if command is executed without problem.
        """
        self.connection.writeRaw("flow 1")

    def airFlowOff(self):
        """
        This function disables the flowing of the air from the thermostream.

        Returns:
            bool: this is true if command is executed without problem.
        """
        self.connection.writeRaw("flow 0")

    def tempReady(self):
        """
        This function returns if the wanted temperature is ready to be used.

        Returns:
            bool: this is true if command is executed without problem.
        """
        self.connection.writeRaw("TECR?")
        time.sleep(0.1)
        return int(self.connection.readRaw().strip()) == 1

    def setTemp(self, temperature: float):
        """
        This function sets in which section (ambient, cold or hot) our temperature is, and sets the wanted temperature.
        
        Args:
            temperature (float): this is the wanted temperature, under which the measurements will be done.

        Returns:
            bool: this is true if command is executed without problem.
        """
        if temperature >= 15.0 and temperature <= 30.0 : #ambient area
            self.setSectionPoint(1)
        elif temperature < 15.0 : #cold area
            self.setSectionPoint(2)
        elif temperature > 30.0 : #hot area
            self.setSectionPoint(0)
        sleep(1)
        self.connection.writeRaw("SETP " + str(temperature))

    def setSoakTime(self, time:int) -> bool:
        """
        This function sets the soak time, the time in which the tempperature is constantly in the acceptable toleranze.

        Args:
            time (int): time to soak in the seconds.

        Returns:
            bool: this is true if command is executed without problem.
        """
        self.connection.writeRaw("SOAK " + str(time))

    def start(self) -> bool:
        """
        This function starts the operation of the thermostream.

        Returns:
            bool: this is true if command is executed without problem.
        """
        self.headDown()
        self.headLock()
        self.airFlowOn()

    def stop(self):
        """
        This function stops the operation of the thermostream.

        Returns:
            bool: this is true if command is executed without problem.
        """
        self.headUnlock()
        self.headUp()

    def goto(self, temperature:float, soak:int) -> bool:
        """
        This function is used to check wether the thermostream has been done with the preparation and is ready to use.

        Args:
            temperature (float): this is the wanted temperature, under which the measurements will be done.
            soak (time): this is the soaking time.

        Returns:
            bool: this is true if command is executed without problem.
        """
        readyFlag = True
        wd_counter = 0
        actualTemperature = 0

        self.setTemp(temperature)
        time.sleep(0.5)
        self.setSoakTime(soak)

        while not self.tempReady():
            print(f"Temp: {self.readTemp()} deg-C")
            sleep(1)
            wd_counter += 1
            if wd_counter > 3600:
                print("Temperature is not reached within 1 hour...")
                return False
        print(f"Temperature is ready at {self.readTemp()}")
        return True

    def reset(self) -> str:
        """
        Used to reset the thermostream.

        Returns:
            str: the reset command.
        """
        return self.connection.writeRaw("*RST")

class commands(Enum):
    START = "start()"
    STOP = "stop()"
    TEMP = "readTemp()"
    HEAD_UP = "headUp()"
    LOCK = "headLock()"
    UNLOCK = "headUnlock()"
    FLOW_ON = "airFlowOn()"
    FLOW_OFF = "airFlowOff()"
    HEAD_DOWN = "headDown()"
    GOTO = "goto()"


if __name__ == '__main__':

    act_commands = []
    arg_idxs = []
    for act_idx, act_arg in enumerate(sys.argv):
        if act_arg.upper() in dir(commands):
            act_commands.append(act_arg.upper())
            arg_idxs.append(act_idx)

    if len(act_commands) > 0:
        termo = ams_temperature_control()
        for idx, act_command in enumerate(act_commands):
            if act_command != "GOTO":
                a_comm = eval("commands.{comm}.value".format(comm=act_command))
                print(eval(f"termo.{a_comm}"))
                
            else:
                termo.goto(float(sys.argv[idx + 2]), float(sys.argv[idx + 3]))
